/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: basicfuncs.c,v 1.6 2019/08/14 20:29:03 piru Exp $
 
    Desc: Functions for readin .font files
    Lang: English.
*/

/****************************************************************************************/

#include <dos/dos.h>
#include <utility/tagitem.h>
#include <proto/exec.h>
#include <proto/utility.h>

#include <aros/debug.h>

#include "diskfont_intern.h"

#include <stddef.h>

/****************************************************************************************/

struct segmentstart
{
	ULONG segmentsize;
	BPTR  nextsegmentptr;
	char  segmentdata[0];
};

APTR AllocSegment(APTR *prevsegment_ptr, ULONG segmentsize, ULONG memflags,
                  struct DiskfontBase_intern *DiskfontBase)
{
	struct segmentstart *seg;
	char *prevsegmentp;

	segmentsize += sizeof(*seg);

	seg = AllocMem(segmentsize, memflags);
	if (!seg)
		return NULL;

	seg->segmentsize    = segmentsize;
	seg->nextsegmentptr = 0;

	if ((prevsegmentp = *prevsegment_ptr))
	{
		struct segmentstart *prevsegment = (void *) (prevsegmentp - offsetof(struct segmentstart, segmentdata));

		prevsegment->nextsegmentptr = MKBADDR(&seg->nextsegmentptr);
	}

	*prevsegment_ptr = seg->segmentdata;

	return seg->segmentdata;
}

/****************************************************************************************/

VOID FreeSegList(BPTR seglist, struct DiskfontBase_intern *DiskfontBase)
{
	ULONG *seg;

	while (seglist)
	{
		seg = BADDR(seglist);

		seglist = *((BPTR *) seg);

		FreeMem(&seg[-1], seg[-1]);
	}
}

/****************************************************************************************/

BPTR AppendToSegList(BPTR seglist, BPTR append)
{
	if (seglist)
	{
		BPTR segment = seglist;
		ULONG *seg;
		do
		{
			seg = BADDR(segment);

			segment = *(BPTR *) seg;
		} while (segment);

		*(BPTR *) seg = append;
	}

	return seglist;
}

/****************************************************************************************/

struct TagItem *ReadTags(BPTR fh, ULONG numtags, struct DiskfontBase_intern *DiskfontBase)
{
	struct TagItem  *taglist,
				*tagptr;

	D(bug("ReadTags(fh=%p, numtags=%d)\n", fh, numtags));

	/* Allocate memory for the tags */

	taglist = AllocateTagItems(numtags);

	if (!taglist)
		goto rt_failure;

	tagptr = taglist;

	/* Read the taglist into the buffer */

	for (; numtags --; )
	{
		ULONG val;

		if (!ReadLong( &DFB(DiskfontBase)->dsh, &val, (void *)fh))
			goto readfail;
		tagptr->ti_Tag = val;

		if (!ReadLong( &DFB(DiskfontBase)->dsh, &val, (void *)fh ))
			goto readfail;
		tagptr->ti_Data = val;

		tagptr ++;
	}

	ReturnPtr ("ReadTags", struct TagItem*, taglist);

readfail:
	FreeTagItems(taglist);
rt_failure:
	ReturnPtr("ReadTags", struct TagItem*, FALSE);

}

/****************************************************************************************/

BOOL WriteTags(BPTR fh, struct TagItem *taglist, struct DiskfontBase_intern *DiskfontBase)
{

	struct TagItem *tag;

	D(bug("WriteTags(fh=%p, taglists=%p)\n", fh, taglist));

#ifdef AROSAMIGA
	for (; (tag = NextTagItem((struct TagItem **)&taglist)); )
#else
	for (; (tag = NextTagItem((const struct TagItem **)&taglist)); )
#endif
	{
		if (!WriteLong( &DFB(DiskfontBase)->dsh, tag->ti_Tag, (void *)fh ))
			goto wt_failure;

		if (!WriteLong( &DFB(DiskfontBase)->dsh, tag->ti_Data, (void *)fh))
			goto wt_failure;
	}
	WriteLong(&DFB(DiskfontBase)->dsh, TAG_DONE, (void *)fh);
	WriteLong(&DFB(DiskfontBase)->dsh, 0, (void *)fh);

	ReturnBool ("WriteTags", TRUE);

wt_failure:
	ReturnBool ("WriteTags", FALSE);
}

/****************************************************************************************/
